/* 
 * Copyright (c) 2005-2006 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

//#include <GL/gl.h>
#include "canvas.h"
#include "scene.h"

Canvas::Canvas (Scene *scene_, bool fullscreen_, int fps_)
{
    scene		= scene_;
    fullscreen	= fullscreen_;
    fps			= fps_;
    refresh		= true;
    active      = true;
    width 		= 0;
    height		= 0;
    panoramic   = false;
}

Canvas::~Canvas (void)
{}


void Canvas::resize(void)
{
    if (panoramic) {
        glViewport    (0, height/4, width, height/2);
        scene->resize (width, height/2);
    } else {
        glViewport    (0, 0, width, height);
        scene->resize (width, height);
    }
}


void Canvas::render (float dt)
{
    scene->setup ();
    scene->render (dt);
}

int Canvas::loop(void)
{
    bool done = false;

    if (panoramic)
        resize();
    
    while (!done) {
        int status = process_events ();
        if (!status)
            break;

        float dt = wait_next_tick();

        if (active) {
            glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
            render (dt);
        } else if (refresh) {
            glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
            render (0);
        }
    }

    return 1;
}

float Canvas::wait_next_tick (void)
{
    if (!active) {
        last_tick = SDL_GetTicks();
        return 0;
    }
    

    if (last_tick == 0) {
        last_tick = SDL_GetTicks();
        return 0;
    }
    
    Uint32 wait      = (Uint32)(1000.0f/fps);
    Uint32 new_tick  = SDL_GetTicks();

    if ((new_tick - last_tick) < wait)
        SDL_Delay (wait - (new_tick-last_tick));

    float dt = (SDL_GetTicks() - last_tick)/1000.0f;
    last_tick = SDL_GetTicks();
    return dt;
}
