/* 
 * Copyright (c) 2005-2006 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <sys/time.h>
#include <time.h>
#include "vroot.h"
#include "canvas_glx.h"


CanvasGLX::CanvasGLX (Scene *scene_, bool fullscreen_, int fps_, Window wid)
    : Canvas (scene_, fullscreen_, fps_)
{
    window = 0;
    display = 0;
    window_id = wid;
}

CanvasGLX::~CanvasGLX (void)
{}

int
CanvasGLX::init (void)
{
    if ((display = XOpenDisplay(0)) == 0)
        return -1;
    screen = DefaultScreen(display);

    window = window_id ? window_id : RootWindow(display, screen);
    XWindowAttributes wa;
    XGetWindowAttributes(display, window, &wa);

    Visual *visual = wa.visual;

    XVisualInfo templ;
    templ.screen   = screen;
    templ.visualid = XVisualIDFromVisual (visual);
    int out_count;

    XVisualInfo *vinfo =
        XGetVisualInfo(display, VisualScreenMask | VisualIDMask, &templ, &out_count);

//    int attributes[] = {GLX_RGBA, None };
//    XVisualInfo *vinfo =
//        glXChooseVisual (display, DefaultScreen(display), attributes);
//    printf ("%p\n", vinfo);
//    printf ("%d %d\n", screen, vinfo->screen);
//    printf ("%ld %ld\n", XVisualIDFromVisual (visual), vinfo->visualid);


    if (!vinfo) {
        std::cerr << "Cannot get Visual information" << std::endl;
        return -1;
    }
    

    GLXContext context = glXCreateContext (display, vinfo, 0, GL_TRUE);
    XFree (vinfo); 
    if (!context) {
        std::cerr << "Cannot create GLX context" << std::endl;
        return -1;
    }
    

    if (!glXMakeCurrent (display, window, context)) {
        std::cerr << "Cannot attach GLX context to window" << std::endl;
        return -1;
    }
    

    XMapRaised (display, window);
    resize ();
    return 0;
}

void
CanvasGLX::resize(void)
{
    XWindowAttributes attrib;
    XGetWindowAttributes(display, window, &attrib);
    width = attrib.width;
    height = attrib.height;
    Canvas::resize ();
}

void
 CanvasGLX::render (float dt)
{
    Canvas::render (dt);
    glXSwapBuffers (display, window);
}

int
CanvasGLX::process_events (void)
{
    XEvent event;
    while (XPending(display)) {
        XNextEvent(display, &event);
        if (event.xany.type == ConfigureNotify)
            resize();
    }
    
    return 1;
}

