/* 
 * Copyright (c) 2005-2006 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include "canvas_sdl.h"


CanvasSDL::CanvasSDL (Scene *scene_, bool fullscreen_, int fps_, char *wm_title_, char *wm_class_)
    : Canvas (scene_, fullscreen_, fps_)
{
    wm_title = wm_title_;
    wm_class = wm_class_;
    surface = 0;
    last_tick = 0;
}

CanvasSDL::~CanvasSDL (void)
{}

int
CanvasSDL::init (void)
{
    flags = SDL_HWSURFACE 
        | SDL_GL_DOUBLEBUFFER
        | SDL_HWPALETTE
        | SDL_HWACCEL
        | SDL_RESIZABLE
        | SDL_OPENGL;


    int status;
    if ((status=SDL_Init(SDL_INIT_VIDEO)) < 0)
        return status;

    if (fullscreen)
        flags |= SDL_FULLSCREEN;

    SDL_Rect **modes;
    modes = SDL_ListModes(0, flags);
    if (modes == (SDL_Rect**)0) {
        std::cerr << "CanvasSDL ERROR: no modes available" << std::endl;
        return -1;
    }
    if (modes == (SDL_Rect**)-1) {
        width = 1024;
        height = 768;
    }
    else {
        width = modes[0]->w;
        height = modes[0]->h;
    }

    surface = SDL_SetVideoMode (width, height, 24, flags);
    if (!surface)
        return -1;
    SDL_WM_SetCaption(wm_title, wm_class);

    resize ();

    return 0;
}

void
CanvasSDL::resize(void)
{
    width = surface->w;
    height = surface->h;
    Canvas::resize ();
}

void CanvasSDL::render (float dt)
{
    Canvas::render (dt);
    SDL_GL_SwapBuffers ();
}

int CanvasSDL::process_events()
{
    SDL_Event event;
    while (SDL_PollEvent(&event)) {
        switch (event.type) {
        case SDL_VIDEOEXPOSE:
            refresh = true;
            break;

        case SDL_VIDEORESIZE:
            refresh = true;
            surface = SDL_SetVideoMode (event.resize.w, event.resize.h, 24, flags);
            if (!surface) {
                std::cerr << "CanvasSDL ERROR: cannot set video mode after resize" << std::endl;
                SDL_Quit();
                return 0;
            }
            resize ();
            break;
        case SDL_KEYDOWN:
            if (event.key.keysym.sym == SDLK_ESCAPE) {
                SDL_Quit ();
                return 0;
            }
            
            break;
            
        case SDL_QUIT:
            SDL_Quit();
            return 0;
            break;
        }
    }
    return 1;
}


