/* 
 * Copyright (c) 2005-2006 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <cmath>
#include "scene.h"


Scene::Scene (void)
{
    speed = 25;
    gridsize = 255;
}

Scene::~Scene (void)
{}

void Scene::init (void)
{  
    radius = 25;
    up = Vec4f (0,1,0, 1);
    eye = Vec4f (cos(0)*radius, 3.25, sin(0)*radius, 1);
    center = Vec4f (0, 1.35,0, 1);

    
    glEnable (GL_TEXTURE_2D);
    glShadeModel (GL_SMOOTH);
    glClearDepth (1.0f);
    glEnable (GL_DEPTH_TEST);
    glDepthFunc (GL_LEQUAL);
    glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glDisable (GL_LIGHTING);

    // WARNING: Replace absolute path with the right one
    hf = new HeightField (DATADIR"/heightmap.png",
                          DATADIR"/lightmap.png",
                          DATADIR"/grass.jpg");
    hf->make (gridsize);


/*
    eye = Vec4f (cos(0)*radius, 3.25, sin(0)*radius, 1);
    center = Vec4f (0, 2.25,0, 1);

    glEnable (GL_FOG);
    glFogi   (GL_FOG_MODE, GL_EXP2);

    float color[] = {.8,.9,1,0};
    float density = .05;

    glFogfv (GL_FOG_COLOR, color);
    glFogf  (GL_FOG_DENSITY, density);
    glHint  (GL_FOG_HINT, GL_NICEST);
*/

    float color[] = {0,0,0,0};
    glClearColor (color[0], color[1], color[2], 0);
}

void Scene::resize(int width, int height)
{
    float ratio = width/float(height);

    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective (30.0f, ratio, 1.0f, 100.0f);
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
}

void Scene::setup (void)
{
    glLoadIdentity ();
    gluLookAt (eye.x, eye.y, eye.z,
               center.x, center.y, center.z,
               up.x, up.y, up.z);
}

void Scene::render (float dt)
{
    if (dt > 1)
        dt = 0;

    static float time = 0.0f;
    time += speed*dt/1000;

    float t = time;
    float dx = cos(    (t+0.5))*radius - eye.x;
    float dz = sin(1.7*(t+0.5))*radius - eye.z;
    float d = sqrt (dx*dx+dz*dz);
    dx /= d;
    dz /= d;
    
    eye.x    = cos(    t)*radius;
    eye.z    = sin(1.7*t)*radius;
    center.x = eye.x + dx*5;
    center.z = eye.z + dz*5;

    glPushMatrix();
    {
        glScalef (100.0f, 3.75f, 100.0f);
        hf->render();
    }
    glPopMatrix ();

}
